==> node-red-nodes-cf-sqldb-dashdb-0.1.0.tgz: package/98-sqldb-dashdb-cf.js <==
/**
 * Copyright 2014 IBM Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
module.exports = function(RED) {

    var cfEnv = require("cfenv");
    var appEnv = cfEnv.getAppEnv();
    var debugLocal = false;
    var SQLDBservices = [];
    var dashDBservices = [];

    // load the services bound to this application
    function extractProperties(v) {
      return { name: v.name, label: v.label };
    }

    for (var i in appEnv.services) {
        // filter for SQLDB services
        if (i.match(/^(sqldb)/i)) {
            SQLDBservices = SQLDBservices.concat(appEnv.services[i].map(extractProperties));
        }
        // filter for dashDB services
        else if (i.match(/^(Analytics)/i)) {
            dashDBservices = dashDBservices.concat(appEnv.services[i].map(extractProperties));
        }
    }

    //
    // HTTP endpoints that will be accessed from the HTML file
    //
    RED.httpAdmin.get('/sqldb/vcap', function(req,res) {
        res.send(JSON.stringify(SQLDBservices));
    });
    
    RED.httpAdmin.get('/dashDB/vcap', function(req,res) {
        res.send(JSON.stringify(dashDBservices));
    });

    function SQLDBOutNode(n) {

        RED.nodes.createNode(this,n);
        var db              = require('ibm_db')();
        this.table          = n.table;
	this.values         = n.values;
	this.insertTS       = n.insertTS || false;
        var SQLDBconfig     = appEnv.getService(n.service);

        if (!this.table) {
	    this.error("SQLDB node configuration error: table not defined");
	    return;
	}

	if (!this.values) {
   	   this.error("SQLDB node configuration error: values not defined");
	   return;
	}

	if (!SQLDBconfig) {
	   this.error("SQLDB node configuration error: service not defined");
	   return; 
	}

	var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = SQLDBconfig.credentials.db;               
           db2.username = SQLDBconfig.credentials.username;
           db2.hostname = SQLDBconfig.credentials.hostname;
           db2.password = SQLDBconfig.credentials.password;
           db2.port = SQLDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("SQLDB: Opening db connection...");
           db.openSync(connString);
           console.log("SQLDB connection open");
	   }
	catch (e) {
           node.error(e.message);
	}

        node.on("close", function() {
        console.log("SQLDB: Closing db connection...");
        db.closeSync();
        console.log("SQLDB: Connection closed");
        });

        console.log("node.values = " + node.values);  
        var questionMarks = genQuestionMarks(node.values);
	if (this.insertTS) {
           var insertStatement = "insert into "+node.table+" values(CURRENT_TIMESTAMP,"+questionMarks+")";
        }	
	else {
           var insertStatement = "insert into "+node.table+" values("+questionMarks+")";
	}
        console.log("SQLDB: Preparing insert statement: " + insertStatement);	
   
        try {
           var stmt = db.prepareSync(insertStatement);
           console.log("SQLDB: Prepare successful");
        } catch (e) {
           node.error(e.message);
        }

        var path = pathToArray(node.values.toString());
        console.log("SQLDB: pathToArray: " + path);

        node.on("input", function(msg) {processInput(msg,path,db,stmt,"SQLDB");});
     }

    function dashDBOutNode(n) {

        RED.nodes.createNode(this,n);

        var db              = require('ibm_db')();
        this.table          = n.table;
	this.values         = n.values;
	this.insertTS       = n.insertTS || false;
        var dashDBconfig    = appEnv.getService(n.service);

        if (!this.table) {
	    this.error("dashDB node configuration error: table not defined");
	    return;
	}

	if (!this.values) {
   	   this.error("dashDB node configuration error: values not defined");
	   return;
	}

	if (!dashDBconfig) {
	   this.error("dashDB node configuration error: service not defined");
	   return; 
	}

	var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = dashDBconfig.credentials.db;               
           db2.username = dashDBconfig.credentials.username;
           db2.hostname = dashDBconfig.credentials.hostname;
           db2.password = dashDBconfig.credentials.password;
           db2.port = dashDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
              db: "TESTDB",
              hostname: "localhost",
              port: 50000,
              username: "db2admin",
              password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("dashDB: Opening db connection...");
           db.openSync(connString);
           console.log("dashDB: Connection open");
	}
	catch (e) {
           node.error(e.message);
	}

        node.on("close", function() {
           console.log("dashDB: Closing db connection...");
           db.closeSync();
           console.log("dashDB: Connection closed");
        });

        console.log("node.values = " + node.values);  
        var questionMarks = genQuestionMarks(node.values);
	if (this.insertTS) {
           var insertStatement = "insert into "+node.table+" values(CURRENT_TIMESTAMP,"+questionMarks+")";
        }	
	else {
           var insertStatement = "insert into "+node.table+" values("+questionMarks+")";
	}
        console.log("dashDB: Preparing insert statement: " + insertStatement);	
   
        try {
           var stmt = db.prepareSync(insertStatement);
           console.log("dashDB: Prepare successful");
        } catch (e) {
           node.error(e.message);
        }

        var path = pathToArray(node.values.toString());
        console.log("dashDB: pathToArray: " + path);

        node.on("input", function(msg) {processInput(msg,path,db,stmt,"dashDB");});
     }

RED.nodes.registerType("sqldb out", SQLDBOutNode);
RED.nodes.registerType("dashDB out", dashDBOutNode);

function processInput (msg,path,db,stmt,service) {
      console.log(service+": Input event received");
      var valuesToInsert = extractValues(msg, path); 
      console.log(service+": Value of valuesToInsert: " + valuesToInsert);
      stmt.execute(valuesToInsert, function (err, result) {
         if (err) {
            console.log(service+": Insert failed: "+err); 
         } else {
            console.log(service+": Insert successful!"); 
            db.commitTransaction(function(err){
               if (err) {
                  console.log(service+": Error during commit: " + err);
               }
               else {
                  console.log(service+": Commit successful");
               }
            });
            result.closeSync();
         }
      });
   }

function pathToArray(commaSeperatedPaths) {
   var pathList = commaSeperatedPaths.split(",");
   var resultArray = new Array(pathList.length);
   for (var i = 0; i < pathList.length; i++) {
      var pathString = pathList[i];
      var fields = pathString.split(".");
      resultArray[i] = fields;
   }
   return resultArray;
}


function extractValues(data, targets) {
   // dbresult will store all resulting targets
   data = {"msg":data};
   var dbresult = [];
   // loop over all targets
   for (var target_index=0; target_index<targets.length; target_index++) {
      var value = data;
      var target = targets[target_index];
      // loop over all fields in target
      for (var field_index=0; field_index<target.length; field_index++) {
         var field = target[field_index];
         // Final value will be the result
         // from the nested data structure
         value = value[field];
      }
      // append value to results
      dbresult.push(value);
   }
   return dbresult;
}

function genQuestionMarks(string) {
	var count = string.split(",").length;
	var resultString = "";
	for (var i = 0; i < count; i++) {
		resultString += "?";
		if (i < count - 1) {
			resultString += ", ";
		}
	}
	return resultString;
}

function SQLDBQueryNode(n) {

        RED.nodes.createNode(this,n);
        var db              = require('ibm_db')();
        this.query          = n.query;
        this.params          = n.params;
        var SQLDBconfig     = appEnv.getService(n.service);

        if (!SQLDBconfig) {
           this.error("SQLDB node configuration error: service not defined");
           return;
        }

        var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = SQLDBconfig.credentials.db;
           db2.username = SQLDBconfig.credentials.username;
           db2.hostname = SQLDBconfig.credentials.hostname;
           db2.password = SQLDBconfig.credentials.password;
           db2.port = SQLDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("SQLDB query node: Opening db connection...");
           db.openSync(connString);
           console.log("SQLDB query node: connection open");
           }
        catch (e) {
           node.error(e.message);
        }

        node.on("close", function() {
           console.log("SQLDB query node: Closing db connection...");
           db.closeSync();
           console.log("SQLDB query node: Connection closed");
        });


        this.on('input', function(msg) {
           if (this.params != "" && this.params != null) {
              var path = pathToArray(this.params.toString());
              console.log("Input node: pathToArray: " + path);
              var parameterValues = extractValues(msg, path);
              console.log("Input node: parameterValues: " + parameterValues);
           }
           else {parameterValues=[];}
           db.query(this.query,parameterValues,function (err, rows, moreResultSets) {
              if (err) {
                 console.log("Error fetching rows: " + err);
              } else {
                 console.log("Fetching rows: " + rows);
                 console.log("value 1: " + JSON.stringify(rows[0]));
                 if (rows.length == 1) {queryresult = rows[0];}
                 else {
                    queryresult = []; 
                    for (var i = 0; i < rows.length; i++) {
                       queryresult.push(rows[i]);
                    }
                 }
            msg.sqldb = queryresult;
            node.send(msg);
           }
        });
      });
    }
    RED.nodes.registerType("sqldb in",SQLDBQueryNode);


function dashDBQueryNode(n) {

        RED.nodes.createNode(this,n);
        var db              = require('ibm_db')();
        this.query          = n.query;
        this.params          = n.params;
        var dashDBconfig     = appEnv.getService(n.service);

        if (!dashDBconfig) {
           this.error("dashDB query node configuration error: service not defined");
           return;
        }

        var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = dashDBconfig.credentials.db;
           db2.username = dashDBconfig.credentials.username;
           db2.hostname = dashDBconfig.credentials.hostname;
           db2.password = dashDBconfig.credentials.password;
           db2.port = dashDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("dashDB query node: Opening db connection...");
           db.openSync(connString);
           console.log("dashDB query node: Connection open");
           }
        catch (e) {
           node.error(e.message);
        }

        node.on("close", function() {
           console.log("dashDB query node: Closing db connection...");
           db.closeSync();
           console.log("dashDB query node: Connection closed");
        });


        this.on('input', function(msg) {
           if (this.params != "" && this.params != null) {
              var path = pathToArray(this.params.toString());
              console.log("Input node: pathToArray: " + path);
              var parameterValues = extractValues(msg, path);
              console.log("Input node: parameterValues: " + parameterValues);
           }
           else {parameterValues=[];}
           db.query(this.query,parameterValues,function (err, rows, moreResultSets) {
              if (err) {
                 console.log("Error fetching rows: " + err);
              } else {
                 console.log("Fetching rows: " + rows);
                 console.log("value 1: " + JSON.stringify(rows[0]));
                 if (rows.length == 1) {queryresult = rows[0];}
                 else {
                    queryresult = []; 
                    for (var i = 0; i < rows.length; i++) {
                       queryresult.push(rows[i]);
                    }
                 }
            msg.dashdb = queryresult;
            node.send(msg);
           }
        });
      });
    }
  RED.nodes.registerType("dashDB in",dashDBQueryNode);

}

==> node-red-nodes-cf-sqldb-dashdb-0.2.1.tgz: package/98-sqldb-dashdb-cf.js <==
/**
 * Copyright 2014 IBM Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
module.exports = function(RED) {

    var cfEnv = require("cfenv");
    var appEnv = cfEnv.getAppEnv();
    var debugLocal = false;
    var SQLDBservices = [];
    var dashDBservices = [];

    // load the services bound to this application
    function extractProperties(v) {
      return { name: v.name, label: v.label };
    }

    for (var i in appEnv.services) {
        // filter for SQLDB services
        if (i.match(/^(sqldb)/i)) {
            SQLDBservices = SQLDBservices.concat(appEnv.services[i].map(extractProperties));
        }
        // filter for dashDB services
        else if (i.match(/^(Analytics)/i)) {
            dashDBservices = dashDBservices.concat(appEnv.services[i].map(extractProperties));
        }
    }

    //
    // HTTP endpoints that will be accessed from the HTML file
    //
    RED.httpAdmin.get('/sqldb/vcap', function(req,res) {
        res.send(JSON.stringify(SQLDBservices));
    });
    
    RED.httpAdmin.get('/dashDB/vcap', function(req,res) {
        res.send(JSON.stringify(dashDBservices));
    });

    function SQLDBOutNode(n) {

        RED.nodes.createNode(this,n);

        var db              = require('ibm_db')();
        this.table          = n.table;
        var SQLDBconfig     = appEnv.getService(n.service);

        if (!this.table) {
	    this.error("SQLDB node configuration error: table not defined");
	    return;
	}

	if (!SQLDBconfig) {
	   this.error("SQLDB node configuration error: service not defined");
	   return; 
	}

	var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = SQLDBconfig.credentials.db;               
           db2.username = SQLDBconfig.credentials.username;
           db2.hostname = SQLDBconfig.credentials.hostname;
           db2.password = SQLDBconfig.credentials.password;
           db2.port = SQLDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("SQLDB output node: Opening db connection...");
           db.openSync(connString);
           console.log("SQLDB output node: Connection open");
	   }
	catch (e) {
           node.error(e.message);
	}

        var columnList = getColumns(node,db,node.table,"SQLDB output node");
        console.log("SQLDB output node: columnList: " + columnList);

        node.on("close", function() {
        console.log("SQLDB output node: Closing db connection...");
        db.closeSync();
        console.log("SQLDB output node: Connection closed");
        });

        console.log("columnList: " + columnList);  
        var questionMarks = genQuestionMarks(columnList);
        
        var insertStatement = "insert into "+node.table+"(" + columnList + ") values("+questionMarks+")";
        console.log("SQLDB output node: Preparing insert statement: " + insertStatement);	
        
        try {
           var stmt = db.prepareSync(insertStatement);
           console.log("SQLDB output node: Prepare successful");
        } catch (e) {
           node.error(e.message);
        }

        node.on("input", function(msg) {processInput(node,msg,db,stmt,columnList,"SQLDB");});
     }

    function dashDBOutNode(n) {

        RED.nodes.createNode(this,n);

        var db              = require('ibm_db')();
        this.table          = n.table;
        var dashDBconfig    = appEnv.getService(n.service);

        if (!this.table) {
	    this.error("dashDB node configuration error: table not defined");
	    return;
	}

	if (!dashDBconfig) {
	   this.error("dashDB node configuration error: service not defined");
	   return; 
	}

	var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = dashDBconfig.credentials.db;               
           db2.username = dashDBconfig.credentials.username;
           db2.hostname = dashDBconfig.credentials.hostname;
           db2.password = dashDBconfig.credentials.password;
           db2.port = dashDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
              db: "TESTDB",
              hostname: "localhost",
              port: 50000,
              username: "db2admin",
              password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("dashDB output node: Opening db connection...");
           db.openSync(connString);
           console.log("dashDB output node: Connection open");
	}
	catch (e) {
           node.error(e.message);
	}

        var columnList = getColumns(node,db,node.table,"dashDB output node");
        console.log("dashDB output node: columnList: " + columnList);

        node.on("close", function() {
           console.log("dashDB: Closing db connection...");
           db.closeSync();
           console.log("dashDB output node: Connection closed");
        });

        var questionMarks = genQuestionMarks(columnList);
        
        var insertStatement = "insert into "+node.table+"(" + columnList + ") values("+questionMarks+")";
        console.log("dashDB output node: Preparing insert statement: " + insertStatement);	
   
        try {
           var stmt = db.prepareSync(insertStatement);
           console.log("dashDB output node: Prepare successful");
        } catch (e) {
           node.error(e.message);
        }

        node.on("input", function(msg) {processInput(node,msg,db,stmt,columnList,"dashDB");});
     }

RED.nodes.registerType("sqldb out", SQLDBOutNode);
RED.nodes.registerType("dashDB out", dashDBOutNode);

function getColumns (node,db,table,service) {
      //Remove the schema, if it exists, hopefully the table name is unique - need to improve this
      var removeSchema = table.split(".");
      if (removeSchema.length > 1) { table = removeSchema[1]; }
      console.log(service+": Fetching column names for table " + table + "...");
      var sysibmColumns;
      try {
         sysibmColumns = db.querySync("select name from sysibm.syscolumns where tbname = '"+table+"'"); 
      }
      catch (e) {
         node.error("Error fetching column list: " + e.message);
         return -1;
      }

      if (sysibmColumns.length == 0) {
         node.error(service+": table "+table+" not found - is it defined?  Case matters."); 
         return -1;
      }
      var columnList = [];
      for (var i = 0; i < sysibmColumns.length; i++) {
           columnList.push(sysibmColumns[i].NAME);
       }
     return columnList;
     }

function processInput (node,msg,db,stmt,columnList,service) {
      console.log(service+": Input event received");
      console.log(service+": columnList: "+columnList);
      var valueToInsert;
      valueList = [];
      for (var i = 0; i < columnList.length; i++) {
         valueToInsert = msg.payload[columnList[i]];
         if (valueToInsert != undefined) {
            if (valueToInsert == 'TIMESTAMP') {
               valueList.push(genDB2Timestamp());
               } 
            else {
               valueList.push(valueToInsert);
               }
            }
         else {node.error(service+": Cannot find column "+columnList[i]+" in the payload"); return;}
      }
      console.log("Values to execute:");
      console.log(valueList);
      stmt.execute(valueList, function (err, result) {
         if (err) {
            node.error(service+": Insert failed: "+err); 
         } else {
            console.log(service+": Insert successful!"); 
            db.commitTransaction(function(err){
               if (err) {
                  console.log(service+": Error during commit: " + err);
               }
               else {
                  console.log(service+": Commit successful");
               }
            });
            result.closeSync();
         }
      });
   }

function pathToArray(commaSeperatedPaths) {
   var pathList = commaSeperatedPaths.split(",");
   var resultArray = new Array(pathList.length);
   for (var i = 0; i < pathList.length; i++) {
      var pathString = pathList[i];
      var fields = pathString.split(".");
      resultArray[i] = fields;
   }
   return resultArray;
}


function extractValues(data, targets) {
   // dbresult will store all resulting targets
   data = {"msg":data};
   var dbresult = [];
   // loop over all targets
   for (var target_index=0; target_index<targets.length; target_index++) {
      var value = data;
      var target = targets[target_index];
      // loop over all fields in target
      for (var field_index=0; field_index<target.length; field_index++) {
         var field = target[field_index];
         // Final value will be the result
         // from the nested data structure
         value = value[field];
      }
      // append value to results
      dbresult.push(value);
   }
   return dbresult;
}

function genQuestionMarks(colList) {
	var count = colList.length;
	var resultString = "";
	for (var i = 0; i < count; i++) {
		resultString += "?";
		if (i < count - 1) {
			resultString += ", ";
		}
	}
	return resultString;
}

function genDB2Timestamp() {
   var d = new Date();
      return d.getFullYear() + "-" +
      ("00" + (d.getMonth() + 1)).slice(-2) + "-" +
      ("00" + d.getDate()).slice(-2) + " " +
      ("00" + d.getHours()).slice(-2) + ":" +
      ("00" + d.getMinutes()).slice(-2) + ":" +
      ("00" + d.getSeconds()).slice(-2);
}

function SQLDBQueryNode(n) {

        RED.nodes.createNode(this,n);
        var db              = require('ibm_db')();
        var query           = n.query;
        var params          = n.params;
        var SQLDBconfig     = appEnv.getService(n.service);

        if (!SQLDBconfig) {
           this.error("SQLDB node configuration error: service not defined");
           return;
        }

        var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = SQLDBconfig.credentials.db;
           db2.username = SQLDBconfig.credentials.username;
           db2.hostname = SQLDBconfig.credentials.hostname;
           db2.password = SQLDBconfig.credentials.password;
           db2.port = SQLDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("SQLDB query node: Opening db connection...");
           db.openSync(connString);
           console.log("SQLDB query node: connection open");
           }
        catch (e) {
           node.error(e.message);
        }

        node.on("close", function() {
           console.log("SQLDB query node: Closing db connection...");
           db.closeSync();
           console.log("SQLDB query node: Connection closed");
        });


        this.on('input', function(msg) {
           if (query == "" || query == null) {
              query = msg.payload; 
              if (msg.payload == "" || msg.payload == null) {
                 node.error("SQLDB out node: msg.payload is empty!");
                 return;
                 } 
           }
           var parameterValues=[];
           if (this.params != "" && this.params != null) {
              var path = pathToArray(this.params.toString());
              console.log("Input node: pathToArray: " + path);
              parameterValues = extractValues(msg, path);
              console.log("Input node: parameterValues: " + parameterValues);
           }
           db.query(query,parameterValues,function (err, rows, moreResultSets) {
              if (err) {
                 console.log("Error fetching rows: " + err);
              } else {
                 console.log("Fetching rows: " + rows);
                 console.log("value 1: " + JSON.stringify(rows[0]));
                 if (rows.length == 1) {queryresult = rows[0];}
                 else {
                    queryresult = []; 
                    for (var i = 0; i < rows.length; i++) {
                       queryresult.push(rows[i]);
                    }
                 }
            msg.payload = queryresult;
            node.send(msg);
           }
        });
      });
    }
    RED.nodes.registerType("sqldb in",SQLDBQueryNode);


function dashDBQueryNode(n) {

        RED.nodes.createNode(this,n);
        var db              = require('ibm_db')();
        var query           = n.query;
        var params          = n.params;
        var dashDBconfig    = appEnv.getService(n.service);

        if (!dashDBconfig) {
           this.error("dashDB query node configuration error: service not defined");
           return;
        }

        var node = this;
        var db2 = {};
        if (!debugLocal) {
           db2.db = dashDBconfig.credentials.db;
           db2.username = dashDBconfig.credentials.username;
           db2.hostname = dashDBconfig.credentials.hostname;
           db2.password = dashDBconfig.credentials.password;
           db2.port = dashDBconfig.credentials.port;
        }
        // This is for debugging locally
        else {
           db2 = {
           db: "TESTDB",
           hostname: "localhost",
           port: 50000,
           username: "db2admin",
           password: "yourpassword"
           };
        }

        var connString = "DRIVER={DB2};DATABASE=" + db2.db + ";UID=" + db2.username + ";PWD=" + db2.password + ";HOSTNAME=" + db2.hostname + ";port=" + db2.port;

        try {
           console.log("dashDB query node: Opening db connection...");
           db.openSync(connString);
           console.log("dashDB query node: Connection open");
           }
        catch (e) {
           node.error(e.message);
        }

        node.on("close", function() {
           console.log("dashDB query node: Closing db connection...");
           db.closeSync();
           console.log("dashDB query node: Connection closed");
        });


        this.on('input', function(msg) {
           if (query == "" || query == null) {
              if (msg.payload == "" || msg.payload == null) {
                 node.error("dashDB out node: msg.payload is empty!");
                 return;
                 } 
              query = msg.payload; 
           }
           var parameterValues=[];
           if (this.params != "" && this.params != null) {
              var path = pathToArray(this.params.toString());
              console.log("Input node: pathToArray: " + path);
              parameterValues = extractValues(msg, path);
              console.log("Input node: parameterValues: " + parameterValues);
           }
           db.query(query,parameterValues,function (err, rows, moreResultSets) {
              if (err) {
                 console.log("Error fetching rows: " + err);
              } else {
                 console.log("Fetching rows: " + rows);
                 console.log("value 1: " + JSON.stringify(rows[0]));
                 if (rows.length == 1) {queryresult = rows[0];}
                 else {
                    queryresult = []; 
                    for (var i = 0; i < rows.length; i++) {
                       queryresult.push(rows[i]);
                    }
                 }
            msg.payload = queryresult;
            node.send(msg);
           }
        });
      });
    }
  RED.nodes.registerType("dashDB in",dashDBQueryNode);

};
